@@ -57,99 +57,6 @@ def tangent_curve_single_list(obj, param_list, normalize):
     return tuple(ret_vector)
 
 
-def normal_curve_single(obj, u, normalize):
-    """ Evaluates the vector normal to the tangent vector at the input parameter, u.
-
-    The output returns a list containing the starting point (i.e. origin) of the vector and the vector itself.
-
-    :param obj: input curve
-    :type obj: abstract.Curve
-    :param u: parameter
-    :type u: float
-    :param normalize: if True, the returned vector is converted to a unit vector
-    :type normalize: bool
-    :return: a list containing "point" and "vector" pairs
-    :rtype: tuple
-    """
-    # Find 1st derivative
-    ders = obj.derivatives(u, 1)
-
-    # Apply fix on https://github.com/orbingol/NURBS-Python/pull/50#issuecomment-499354073
-    t = ders[1]
-    tn = [0.0, 0.0, 1.0]
-    n = linalg.vector_cross(t, tn)
-
-    # Normalize the vector component
-    vector = linalg.vector_normalize(n) if normalize else n
-    point = ders[0]
-
-    return tuple(point), tuple(vector)
-
-
-def normal_curve_single_list(obj, param_list, normalize):
-    """ Evaluates the curve normal vectors at the given list of parameter values.
-
-    :param obj: input curve
-    :type obj: abstract.Curve
-    :param param_list: parameter list
-    :type param_list: list or tuple
-    :param normalize: if True, the returned vector is converted to a unit vector
-    :type normalize: bool
-    :return: a list containing "point" and "vector" pairs
-    :rtype: tuple
-    """
-    ret_vector = []
-    for param in param_list:
-        temp = normal_curve_single(obj, param, normalize)
-        ret_vector.append(temp)
-    return tuple(ret_vector)
-
-
-def binormal_curve_single(obj, u, normalize):
-    """ Evaluates the curve binormal vector at the given u parameter.
-
-    Curve binormal is the cross product of the normal and the tangent vectors.
-    The output returns a list containing the starting point (i.e. origin) of the vector and the vector itself.
-
-    :param obj: input curve
-    :type obj: abstract.Curve
-    :param u: parameter
-    :type u: float
-    :param normalize: if True, the returned vector is converted to a unit vector
-    :type normalize: bool
-    :return: a list containing "point" and "vector" pairs
-    :rtype: tuple
-    """
-    # Cross product of tangent and normal vectors gives binormal vector
-    tan_vector = tangent_curve_single(obj, u, normalize)
-    norm_vector = normal_curve_single(obj, u, normalize)
-
-    point = tan_vector[0]
-    vector = linalg.vector_cross(tan_vector[1], norm_vector[1])
-    vector = linalg.vector_normalize(vector) if normalize else vector
-
-    return tuple(point), tuple(vector)
-
-
-def binormal_curve_single_list(obj, param_list, normalize):
-    """ Evaluates the curve binormal vectors at the given list of parameter values.
-
-    :param obj: input curve
-    :type obj: abstract.Curve
-    :param param_list: parameter list
-    :type param_list: list or tuple
-    :param normalize: if True, the returned vector is converted to a unit vector
-    :type normalize: bool
-    :return: a list containing "point" and "vector" pairs
-    :rtype: tuple
-    """
-    ret_vector = []
-    for param in param_list:
-        temp = binormal_curve_single(obj, param, normalize)
-        ret_vector.append(temp)
-    return tuple(ret_vector)
-
-
 def tangent_surface_single(obj, uv, normalize):
     """ Evaluates the surface tangent vector at the given (u,v) parameter pair.
 
